import os, json, tempfile
from pathlib import Path

abstra_json = {"version": "0.2"}


def rm_tree(pth: Path):
    pth = Path(pth)
    if not pth.exists():
        return
    for child in pth.glob("*"):
        if child.is_file():
            child.unlink()
        else:
            rm_tree(child)
    pth.rmdir()


def init_dir():
    path = Path(tempfile.mkdtemp())
    abstra_json_path = path / "abstra.json"
    path.mkdir(exist_ok=True)
    if not abstra_json_path.exists():
        abstra_json_path.write_text(json.dumps(abstra_json))
    os.environ["ABSTRA_DATABASE_URL"] = str(Path(path, "db.sqlite3"))
    os.chdir(path)
    return path


def clear_dir(path: Path):
    rm_tree(path)



from abstra_cli.utils import (
    format_digits,
    parse_env_var,
    parse_package,
    digits,
    remove_prefix,
)


class TestUtils:
    def test_remove_prefix(self):
        assert remove_prefix("foo", "foo") == ""
        assert remove_prefix("foo", "bar") == "foo"
        assert remove_prefix("foo", "") == "foo"
        assert remove_prefix("", "") == ""

    def test_digits(self):
        assert digits(123) == 3
        assert digits(0) == 1

    def test_format_digits(self):
        assert format_digits(123, 4) == " 123"
        assert format_digits(0, 6) == "     0"

    def test_parse_env_var(self):
        assert parse_env_var("FOO=bar") == ("FOO", "bar")
        assert parse_env_var("FOO=bar=baz") == ("FOO", "bar=baz")
        assert parse_env_var("FOO") == (None, None)
        assert parse_env_var("FOO=") == (None, None)

    def test_parse_package(self):
        assert parse_package("foo~=1.0") == ("foo", "1.0")
        assert parse_package("foo==1.0") == ("foo", "1.0")
        assert parse_package("foo>=1.0") == ("foo", "1.0")
        assert parse_package("foo<=1.0") == ("foo", "1.0")
        assert parse_package("foo===1.0") == ("foo", "1.0")

        assert parse_package("foo>1.0") == (None, None)
        assert parse_package("foo<1.0") == (None, None)
        assert parse_package("SomeProject == 5.4 ; python_version < '3.8'") == (
            None,
            None,
        )
        assert parse_package("SomeProject >= 1.2, < 2.0") == (None, None)
        assert parse_package("SomeProject[foo, bar]") == (None, None)

        assert parse_package("foo") == ("foo", None)
        assert parse_package("git+https://github.com/abstra.app/abstra-cli.git") == (
            "git+https://github.com/abstra.app/abstra-cli.git",
            None,
        )



import tempfile
import unittest
from uuid import uuid4 as uuid
from pathlib import Path

from abstra_server.api import API

from .fixtures import init_dir, clear_dir


class TestForms(unittest.TestCase):
    def setUp(self) -> None:
        self.root = init_dir()

    def tearDown(self) -> None:
        clear_dir(self.root)

    def updates_across_reloads(self):
        api = API(root=self.root)
        form = api.create_form()

        api.update_form(form.path, dict(title="New Title"))

        api2 = API(root=self.root)
        api2.get_workspace()
        new_form = api2.get_form(form.path)

        self.assertEqual(form.title, new_form.title)

    def test_raise_exception_on_invalid_propery_update(self):
        api = API(root=self.root)
        form = api.create_form()

        with self.assertRaises(Exception):
            api.update_form(
                form.path, dict(title="New Title", invalid_property="invalid")
            )



import unittest
from abstra_server.api import API
from .fixtures import init_dir, clear_dir


class TestSidebar(unittest.TestCase):
    def setUp(self) -> None:
        self.root = init_dir()

    def tearDown(self) -> None:
        clear_dir(self.root)

    def test_starts_empty(self):
        api = API(root=self.root)

        workspace = api.get_workspace()
        self.assertEqual(workspace.__dict__["sidebar"], [])

    def test_auto_add_forms(self):
        # given
        api = API(root=self.root)

        # when
        form = api.create_form()
        workspace = api.get_workspace()

        # then
        sidebar_json = workspace.__dict__["sidebar"]
        self.assertEqual(len(sidebar_json), 1)
        item = dict(name=form.title, path=form.path, type="form", visible=False)
        self.assertEqual({**sidebar_json[0], **item}, sidebar_json[0])

    def test_auto_add_dashes(self):
        # given
        api = API(root=self.root)

        # when
        dash = api.create_dash()
        workspace = api.get_workspace()

        # then
        sidebar_json = workspace.__dict__["sidebar"]
        self.assertEqual(len(sidebar_json), 1)
        item = {"name": dash.title, "path": dash.path, "type": "dash", "visible": False}
        self.assertEqual({**sidebar_json[0], **item}, sidebar_json[0])
